package minischeme

abstract class Instruction(val opcode: Opcode) extends Representable {
  def representation: String =
    "\t" + opcode + " " + (args map { x => x.representation}).mkString(" ")
  protected def args: List[Representable]
}

class InstructionRRR(opcode: OpcodeRRR, var r1: Register, var r2: Register, var r3: Register) extends Instruction(opcode) {
  override def args: List[Representable] = List(r1,r2,r3)
}

class InstructionRR(opcode: ALOC.type, var r1: Register, var r2: Register) extends Instruction(opcode) {
  override def args: List[Representable] = List(r1,r2)
}

class InstructionRC(opcode: LINT.type, var r: Register, var c: Representable) extends Instruction(opcode) {
  override def args: List[Representable] = List(r, c)
}

class InstructionR(opcode: OpcodeR, var r: Register) extends Instruction(opcode) {
  override def args: List[Representable] = List(r)
}

class InstructionHalt(opcode: HALT.type) extends Instruction(opcode) {
  override def args: List[Representable] = List()
}
